 /*****************************************************************************
 *   Copyright (C) 2004 by Michael Schulze                                    *
 *   mike.s@genion.de                                                         *
 *                                                                            *
 *  The code contained in this file is free software; you can redistribute    *
 *  it and/or modify it under the terms of the GNU Lesser General Public      *
 *  License as published by the Free Software Foundation; either version      *
 *  2.1 of the License, or (at your option) any later version.                *
 *                                                                            *
 *  This file is distributed in the hope that it will be useful,              *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         *
 *  Lesser General Public License for more details.                           *
 *                                                                            *
 *  You should have received a copy of the GNU Lesser General Public          *
 *  License along with this code; if not, write to the Free Software          *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 *                                                                            *
 *  iTunes and iPod are trademarks of Apple                                   *
 *                                                                            *
 *  This product is not supported/written/published by Apple!                 *
 *****************************************************************************/

#include "ipodmountpoint.h"

#include <qfile.h>
#include <qtextstream.h>
#include <qstringlist.h>

#ifdef WIN32
#include <qdir.h>
#endif

#include "utils.h"

IPodMountPoint::IPodMountPoint(const StringVector& entry)
    : fields( entry )
{
}


IPodMountPoint::~IPodMountPoint()
{
}


const QString& IPodMountPoint::getField( unsigned int fieldnum ) const {
    if (fields.size() < fieldnum + 1)
        return itunesdb::utils::NullQString;
    else
        return fields[fieldnum];
}

const QString& IPodMountPoint::getDevice() const {
    return getField(0);
}

const QString& IPodMountPoint::getMountPoint() const {
    return getField(1);
}

const QString& IPodMountPoint::getFsType() const {
    return getField(2);
}

const QString& IPodMountPoint::getOptions() const {
    return getField(3);
}

inline bool possiblyAnIpod( IPodMountPoint& mp ) {
    QString device = mp.getDevice();

    if ((device.find("/dev/sd") == 0 || device.find("scsi") >= 0 || // some kinda scsi device
        device.find("ipod") >= 0 || // special device for ipod
        device.endsWith("2") || // is the 2nd (windows ipod) ...
        device.endsWith("3")) &&    // ... or the 3rd (apple) partition
        QFile::exists( mp.getMountPoint() + "/iPod_Control" ) ) {
        return true;
    } else {
        return false;
    }
}


inline void fillInIpodMountPointsLinux( IPodMountPoint::List& list ) {
    QFile procMounts("/proc/mounts");
    // QFile procMounts("/etc/mtab");
    if ( !procMounts.open(IO_ReadOnly) ) {
        return;
    }
    QTextStream instream(&procMounts);

    QString line;
    while (!(line = instream.readLine()).isEmpty()) {
        if (!line.isEmpty()) {
            QStringList fields = QStringList::split( " ", line );
            // quick fix for spaces in mounts entry
            IPodMountPoint::StringVector entries;
            for ( QStringList::iterator iter = fields.begin(); iter != fields.end(); ++iter ) {
                entries.push_back( (*iter).replace( "\\040", " " ) );
            }

            IPodMountPoint mountpoint( entries );
            // only care about "possibly an ipod" devices (filter out network shares and floppies)
            if ( possiblyAnIpod( mountpoint ) ) {
                list.push_back( mountpoint );
            }
        }
    }

    instream.unsetDevice();
    procMounts.close();
}

#ifdef WIN32
inline void fillInIpodMountPointsWin32( IPodMountPoint::List& list ) {
    QFileInfoList drives = QDir::drives();
    QFileInfoList::iterator driveIter = drives.begin();
    for( ; driveIter != drives.end(); ++driveIter ) {
        QFileInfo& drive = *driveIter;
        const QString& driveStr = drive.absolutePath();
        switch ( driveStr.at( 0 ).toAscii() ) {
        case 'A':
        case 'B':
        case 'C':
        	// filter out floppies and the primary harddrive
            continue;
        }

        IPodMountPoint::StringVector fields;
        fields.push_back( driveStr );
        fields.push_back( driveStr );

        list.push_back( IPodMountPoint( fields ) );
    }
}
#endif

IPodMountPoint::List IPodMountPoint::mountedIPods() {
    List list;

#ifdef WIN32
    fillInIpodMountPointsWin32( list );
#else
    fillInIpodMountPointsLinux( list );
#endif

    return list;
}

